<?php


/**
 * @file
 * Bubbletimer admin page
 */


/**
 * Administer setup form
 * @return array
 */
function bubbletimer_admin_settings_form() {
  $node_types = array();
  foreach (node_type_get_types() as $type) {
    $node_types[$type->type] = $type->name;
  }

  $node_type_values = array_filter(array_keys(node_type_get_types()),
    'bubbletimer_node_type_enabled');

  $form['bt_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Bubbletimer global settings'),
  );

  $form['bt_fieldset']['bt_node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Allowed node types'),
    '#options' => $node_types,
    '#default_value' => $node_type_values,
  );

  $form['bt_fieldset']['bt_form_on_node_page_title'] = array(
    '#type' => 'item',
    '#title' => t('Additional add/remove-node form'),
  );

  $form['bt_fieldset']['bt_form_on_node_page'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add/remove-node form on the node view page.'),
    '#default_value' => variable_get('bubbletimer_form_on_node_page', '0'),
  );

  $form['bt_fieldset']['bt_node_refresh'] = array(
    '#type' => 'checkbox',
    '#title' => t('Update node when bubbletimer is saved'),
    '#default_value' => variable_get('bubbletimer_node_refresh', '0'),
    '#description' => t('Ex.: updating nodes can be handy when you use Computed fields to store total Bubbletimer times.'),
  );

  $form['bt_fieldset']['bt_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}


/**
 * Submit event for administer form
 *
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_admin_settings_form_submit($form, $form_state) {
  foreach ((array)$form_state['values']['bt_node_types'] as $node_type => $value) {
    variable_set(sprintf(
      BT_NODE_TYPE_VARIABLE_PATTERN,
      $node_type
    ), ($value !== 0 ? BT_NODE_TYPE_ENABLE : BT_NODE_TYPE_DISABLE));
  }
  variable_set('bubbletimer_form_on_node_page', $form_state['values']['bt_form_on_node_page']);
  variable_set('bubbletimer_node_refresh', $form_state['values']['bt_node_refresh']);
  drupal_set_message(t('Node type configuration is saved.'));
}

