

/**
 * @file
 * Bubbletimer main javascript
 */

try {
  console;
} catch(ex) {
  console = {};
  console.log = function() {};
}

Drupal = Drupal || {};
Drupal.Bubbletimer = Drupal.Bubbletimer || {};

(function ($) {

// How much time the stat update should be delayed
Drupal.Bubbletimer.BT_UPDATE_DELAY_MS = 3000;
Drupal.Bubbletimer.BT_SECONDS_INTERVAL = 15 * 60; // 15 minutes
//SHIFT - for interval selection
Drupal.Bubbletimer.BT_INTERVAL_KEY_CODE = 16;
//CTRL - for multijob selections
Drupal.Bubbletimer.BT_MULTI_KEY_CODE = 17;
Drupal.Bubbletimer.BT_CHECKBOX_SELECTOR = 'table.bt_checkbox div input:checkbox';

Drupal.Bubbletimer.BT_HIGHLIGHT_CLASS = 'bt_title_highlight';

// Finding a checkbox which timestamp its related to
Drupal.Bubbletimer.bt_job_timestamp_regexp = /^bt_job_(\d*)_timestamp_(\d*)$/g;
// Check if keys are down
Drupal.Bubbletimer.interval_key_is_active = false;
Drupal.Bubbletimer.multi_key_is_active = false;
// setTimeout var
Drupal.Bubbletimer.bt_update_delay = 0;
// true if changes occured on the form
Drupal.Bubbletimer.bt_update_need_save_flag = false;
// nodes
Drupal.Bubbletimer.bt_job_array = [];
// true if charts isnt refreshed
Drupal.Bubbletimer.bt_first_reload = true;
// last selected slot
Drupal.Bubbletimer.bubbletimer_last_selected = false;

Drupal.Bubbletimer.bubbletimer_history_backward_array = [];
Drupal.Bubbletimer.bubbletimer_history_forward_array  = [];

Drupal.Bubbletimer.BUBBLETIMER_SLOT_UNSELECT = -1;
Drupal.Bubbletimer.BUBBLETIMER_SLOT_DEFAULT_SELECT = 0;
Drupal.Bubbletimer.BUBBLETIMER_SLOT_SELECT   = 1;
Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE = 1;

// Table slider info
// For boosting, hiding comes from php: @see bubbletimer.page.inc at line 320,358
Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS = 10;
Drupal.Bubbletimer.bt_table_first_visible_hour = 8;
Drupal.Bubbletimer.bt_table_step_value = 1;

// Cache for highlighting
Drupal.Bubbletimer.bt_cache_highlighted_title = null;
Drupal.Bubbletimer.bt_cache_highlighted_hour = null;

Drupal.behaviors.bubbletimer = {
  attach: function () {
    $('div.bt_checkbox').click(function () {
      Drupal.Bubbletimer.bubbletimer_slot_toggle($(this));
    });

    // During page reload browser restore all hidden values but image states
    Drupal.Bubbletimer.bt_restore_selection();

    // Main BT table init
    Drupal.Bubbletimer.bt_init_table();

    // Datepicker init
    Drupal.Bubbletimer.bt_init_datepicker();

    if (Drupal.settings.bubbletimer.piechart_enable ||
        Drupal.settings.bubbletimer.barchart_enable) {
      $('#bt_chart_title').show();
    }

    // Node type selector
    $('#edit-bt-node-type')
      .bind('change', Drupal.Bubbletimer._bubbletimer_node_select_refresh)
      .trigger('change');

    Drupal.Bubbletimer.init_date_selector_form();

    // Attach key event listeners
    $(window).keydown(function(event){
      switch (event.keyCode) {
        case Drupal.Bubbletimer.BT_INTERVAL_KEY_CODE:
          Drupal.Bubbletimer.interval_key_is_active = true;
          Drupal.Bubbletimer.multi_key_is_active = false;
          break;
        case Drupal.Bubbletimer.BT_MULTI_KEY_CODE:
          Drupal.Bubbletimer.multi_key_is_active = true;
          Drupal.Bubbletimer.interval_key_is_active = false;
          break;
        default:
          Drupal.Bubbletimer.interval_key_is_active = false;
          Drupal.Bubbletimer.multi_key_is_active = false;
          break;
      }
    });
    $(window).keyup(function(){
      Drupal.Bubbletimer.interval_key_is_active = false;
      Drupal.Bubbletimer.multi_key_is_active = false;
      Drupal.Bubbletimer.bt_multiselect_fst_item = null;
    });

    // Cache node ids
    $.each($('a.job_node'), function() {
      Drupal.Bubbletimer.bt_job_array.push($(this).attr('nid'));
    });

    // Update stats
    Drupal.Bubbletimer.bt_update_stats();

    Drupal.Bubbletimer.ajaxform.init();
  }
};

Drupal.Bubbletimer._dateobj_convert = function (obj) {
  return new Date(obj.year, obj.month - 1, obj.day);
};

Drupal.Bubbletimer.select_datepicker_date = function (year, month, day) {
  $("select.ui-datepicker-new-year").val(year).trigger('change');
  $("select.ui-datepicker-new-month").val(month - 1).trigger('change');
  $("table.ui-datepicker tr td a").filter(function () {
    return $(this).text() == day;
  }).parent().trigger('click');
  $("#edit-bt-date-datepicker-popup-0").focus();
};

Drupal.Bubbletimer.select_datepicker_date_wrapper = function (obj) {
  with (obj) {
    Drupal.Bubbletimer.select_datepicker_date(year, month, day);
  }
};

Drupal.Bubbletimer.init_date_selector_form = function () {
  $("#edit-bt-date-datepicker-popup-0").trigger('focus');
  // Add actions for date stepper
  $('#edit-bt-date-prev').click(function(event) {
    event.preventDefault();
    Drupal.Bubbletimer.select_datepicker_date_wrapper(
        Drupal.settings.bubbletimer.date_stepper.previous);
  });
  $('#edit-bt-date-next').click(function(event) {
    event.preventDefault();
    Drupal.Bubbletimer.select_datepicker_date_wrapper(
        Drupal.settings.bubbletimer.date_stepper.next);
  });
  $('#edit-bt-date-now').click(function(event) {
    event.preventDefault();
    Drupal.Bubbletimer.select_datepicker_date_wrapper(
        Drupal.settings.bubbletimer.date_stepper.today);
  });
  $('#edit-bt-date-last').click(function(event) {
    event.preventDefault();
    Drupal.Bubbletimer.select_datepicker_date_wrapper(
        Drupal.settings.bubbletimer.date_stepper.last);
  });
  try {
    $.datepicker._defaults.onSelect = function() {
      $("#bubbletimer-date-selector-form").submit();
    }
  } catch(ex) {
    console.log(ex);
  }
};


/**
 * Handle a checkbox click event
 * If CTRL is down, multiple selection allowed, otherwise doesn't
 * @return
 */
Drupal.Bubbletimer.bubbletimer_slot_toggle = function (div) {
  Drupal.Bubbletimer.slot_div = $(div);

  // Setup a notifier to make us save the changes
  if (!Drupal.Bubbletimer.bt_update_need_save_flag) {
    Drupal.Bubbletimer.bt_update_need_save_flag = true;
    $('div#bt_notify_to_save').show();
  }

  // Multi task selection
  if (Drupal.Bubbletimer.multi_key_is_active) {
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(Drupal.Bubbletimer.slot_div);
    return;
  }

  var slot_wrapper = Drupal.Bubbletimer.slot_div.parent();
  var hidden = $(':hidden', slot_wrapper);

  // Interval selection
  if (Drupal.Bubbletimer.interval_key_is_active) {
    if (Drupal.Bubbletimer.bubbletimer_last_selected) {
      var last_hidden = Drupal.Bubbletimer.bubbletimer_last_selected;
      // Finish a multiselection
      var nid = hidden.attr('nid');
      // Only in the same line
      if (last_hidden.attr('nid') == nid) {
        var timestamp_from = Math.min(hidden.attr('timestamp'), last_hidden.attr('timestamp'));
        var timestamp_to   = Math.max(hidden.attr('timestamp'), last_hidden.attr('timestamp'));
        Drupal.Bubbletimer._bubbletimer_select_range(timestamp_from, timestamp_to, nid);
        Drupal.Bubbletimer.bt_update_stats();
      }
    }
    return;
  }

  // Need manual help to unselect a slot
  if (hidden.val() == 1) {
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(Drupal.Bubbletimer.slot_div);
    Drupal.Bubbletimer.bt_update_stats();
    return;
  }

  // Only 1 checked slot in one column
  var timestamp = hidden.attr('timestamp');
  $.each($('div', $("table#bt_table input:hidden[value='1'][timestamp='" + timestamp + "']").parent()), function() {
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(
        this, [Drupal.Bubbletimer.BUBBLETIMER_SLOT_UNSELECT]);
  });

  // Click event
  Drupal.Bubbletimer._bubbletimer_slot_select.apply(Drupal.Bubbletimer.slot_div);
  // Update stat
  Drupal.Bubbletimer.bt_update_stats();
};


/**
 * Loader icon on
 * @return
 */
Drupal.Bubbletimer.bt_throbber_on = function () {
  $('span.bt_chart_updater').addClass('occupied');
};


/**
 * Loader icon off
 * @return
 */
Drupal.Bubbletimer.bt_throbber_off = function () {
  $('span.bt_chart_updater').removeClass('occupied');
};


/**
 * @PRIVATE
 *
 * Proxy event for clicking on a slot
 * The image state has to be syncronized with the checkbox's state
 *
 * @param forced_select
 *  if BUBBLETIMER_SLOT_SELECT then the result will be a selected slot
 *  if BUBBLETIMER_SLOT_UNSELECT then the result will be an unselected slot
 * @param _history_no_save
 *  if BUBBLETIMER_NO_HISTORY_SAVE then the event won't be saved to the history
 *  (it's necessary in a range selection, we want to save range in one peace)
 */
Drupal.Bubbletimer._bubbletimer_slot_select = function (forced_select, _history_no_save) {
  var do_select       = forced_select     === Drupal.Bubbletimer.BUBBLETIMER_SLOT_SELECT;
  var do_unselect     = forced_select     === Drupal.Bubbletimer.BUBBLETIMER_SLOT_UNSELECT;
  var history_no_save = _history_no_save  === Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE;
  var hidden = $(':hidden', $(this).parent());
  if ((hidden.val() == 1 && !do_select) || do_unselect) {
    hidden.val(0);
  }
  else {
    hidden.val(1);
    Drupal.Bubbletimer.bubbletimer_last_selected = hidden;
  }

  if (!history_no_save) {
    Drupal.Bubbletimer._bubbletimer_save_history(hidden);
  }
};


/**
 * @PRIVATE
 *
 * Select a range of slots
 *
 * @param timestamp_from
 * @param timestamp_to
 * @param nid
 */
Drupal.Bubbletimer._bubbletimer_select_range = function (timestamp_from, timestamp_to, nid) {
  console.log(("range " + timestamp_from + ".." + timestamp_to));
  var history = [];
  for (var t = timestamp_from; t <= timestamp_to;
       t += Drupal.Bubbletimer.BT_SECONDS_INTERVAL) {
    var slot_div = $('div', $("table#bt_table input:hidden[timestamp='" + t + "'][nid='" + nid + "']").parent());
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(
        slot_div, [
          Drupal.Bubbletimer.BUBBLETIMER_SLOT_SELECT,
          Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE]);
    history.push(slot_div);
  }
  Drupal.Bubbletimer.bubbletimer_history_backward_array.pop();
  Drupal.Bubbletimer._bubbletimer_save_history(history);
};


/**
 * @PRIVATE
 *
 * Swapping note-type selectors on the [add node] from
 * Nodes should be displayed only from the selected node-type
 */
Drupal.Bubbletimer._bubbletimer_node_select_refresh = function () {
  $('.bubbletimer_node_select').parent().css('display', 'none');
  $('select[node_type="' + $('#edit-bt-node-type').val() + '"]').parent().show();
};

Drupal.Bubbletimer.bt_restore_selection = function () {
  $.each($('div', $("table#bt_table input:hidden[value='1']").parent()), function() {
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(
        this, [Drupal.Bubbletimer.BUBBLETIMER_SLOT_SELECT]);
  });
};

Drupal.Bubbletimer.bt_init_datepicker = function () {
};

Drupal.Bubbletimer.bt_init_table = function () {
  // For boosting, hiding comes from php: @see bubbletimer.page.inc at line 320,358
  //bt_table_visibility_refresh();
  $('#bt_table_step_left').bind('click', Drupal.Bubbletimer.bt_table_step_left);
  $('#bt_table_step_right').bind('click', Drupal.Bubbletimer.bt_table_step_right);

  // Setup title/hour highlighting
  $('.bt_slot_td').mouseover(function() {
    $(Drupal.Bubbletimer.bt_cache_highlighted_title).removeClass('highlight');
    $(Drupal.Bubbletimer.bt_cache_highlighted_hour).removeClass('highlight');

    $("td.bt_title_td[nid='" + $(this).attr('nid') + "']").addClass('highlight');
    $("td.bt_hour_td[hour='" + $(this).attr('hour') + "']").addClass('highlight');

    Drupal.Bubbletimer.bt_cache_highlighted_title =
        $("td.bt_title_td[nid='" + $(this).attr('nid') + "']");
    Drupal.Bubbletimer.bt_cache_highlighted_hour =
        $("td.bt_hour_td[hour='" + $(this).attr('hour') + "']");
  });
  $('.bt_slot_td').mouseout(function() {
    $(Drupal.Bubbletimer.bt_cache_highlighted_title).removeClass('highlight');
    $(Drupal.Bubbletimer.bt_cache_highlighted_hour).removeClass('highlight');
  });
};


Drupal.Bubbletimer.bt_table_visibility_refresh = function () {
  for (var i = 0; i < 24; i++) {
    if (i < Drupal.Bubbletimer.bt_table_first_visible_hour ||
        i >= Drupal.Bubbletimer.bt_table_first_visible_hour +
            Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS) {
      $("table#bt_table td[hour='" + i + "']").hide();
    }
    else {
      $("table#bt_table td[hour='" + i + "']").show();
    }
  }
};


Drupal.Bubbletimer.bt_table_step_left = function () {
  $('#bt_table_step_right').show();
  var old_bt_table_first_visible_hour =
      Drupal.Bubbletimer.bt_table_first_visible_hour;
  Drupal.Bubbletimer.bt_table_first_visible_hour -=
      Drupal.Bubbletimer.bt_table_step_value;
  if (Drupal.Bubbletimer.bt_table_first_visible_hour < 0) {
    Drupal.Bubbletimer.bt_table_first_visible_hour = 0;
  }
  if (Drupal.Bubbletimer.bt_table_first_visible_hour === 0) {
    $('#bt_table_step_left').hide();
  }
  for (var i = Drupal.Bubbletimer.bt_table_first_visible_hour;
       i < old_bt_table_first_visible_hour; i++) {
    $("table#bt_table td[hour='" + i + "']").show();
    $("table#bt_table td[hour='" +
        (i + Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS) + "']").hide();
  }
};


Drupal.Bubbletimer.bt_table_step_right = function () {
  $('#bt_table_step_left').show();
  var old_bt_table_first_visible_hour =
      Drupal.Bubbletimer.bt_table_first_visible_hour;
  Drupal.Bubbletimer.bt_table_first_visible_hour +=
      Drupal.Bubbletimer.bt_table_step_value;
  if (Drupal.Bubbletimer.bt_table_first_visible_hour >
      24 - Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS) {
    Drupal.Bubbletimer.bt_table_first_visible_hour =
        24 - Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS;
  }
  if (Drupal.Bubbletimer.bt_table_first_visible_hour ==
      24 - Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS) {
    $('#bt_table_step_right').hide();
  }
  for (var i = old_bt_table_first_visible_hour;
       i < Drupal.Bubbletimer.bt_table_first_visible_hour; i++) {
    $("table#bt_table td[hour='" + i + "']").hide();
    $("table#bt_table td[hour='" +
        (i + Drupal.Bubbletimer.BUBBLETIMER_TABLE_VISIBLE_HOURS) + "']").show();
  }
};

/**
 * Step 1 action back in history
 */
Drupal.Bubbletimer.bubbletimer_history_backward = function () {
  if (Drupal.Bubbletimer.bubbletimer_history_backward_array.length === 0) return;

  var oImages = Drupal.Bubbletimer._bubbletimer_get_history();
  Drupal.Bubbletimer._bubbletimer_save_forward(oImages);
  if (oImages.length > 1) {
    for (var idx in oImages) {
      Drupal.Bubbletimer._bubbletimer_slot_select.apply(oImages[idx],
        [Drupal.Bubbletimer.BUBBLETIMER_SLOT_DEFAULT_SELECT,
          Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE]);
    }
  }
  else {
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(oImages,
      [Drupal.Bubbletimer.BUBBLETIMER_SLOT_DEFAULT_SELECT,
        Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE]);
  }
};


/**
 * Step 1 action forward in history
 */
Drupal.Bubbletimer.bubbletimer_history_forward = function () {
  if (Drupal.Bubbletimer.bubbletimer_history_forward_array.length === 0) return;

  var oImages = Drupal.Bubbletimer._bubbletimer_get_forward();
  Drupal.Bubbletimer.bubbletimer_history_backward_array.push(oImages);
  if (oImages.length > 1) {
    for (var idx in oImages) {
      Drupal.Bubbletimer._bubbletimer_slot_select.apply(oImages[idx],
        [Drupal.Bubbletimer.BUBBLETIMER_SLOT_DEFAULT_SELECT,
          Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE]);
    }
  }
  else {
    Drupal.Bubbletimer._bubbletimer_slot_select.apply(oImages,
      [Drupal.Bubbletimer.BUBBLETIMER_SLOT_DEFAULT_SELECT,
        Drupal.Bubbletimer.BUBBLETIMER_NO_HISTORY_SAVE]);
  }
};

/**
 * Save a slot to the history
 *
 * @param {Object} history
 */
Drupal.Bubbletimer._bubbletimer_save_history = function (history) {
  Drupal.Bubbletimer.bubbletimer_history_backward_array.push(history);
  // If a new slot marked/unmarked, the redo history (future) has to be erased
  Drupal.Bubbletimer.bubbletimer_history_forward_array = [];
};


/**
 * Get the latest action from history (past)
 *
 * @return {Object}
 */
Drupal.Bubbletimer._bubbletimer_get_history = function () {
  return Drupal.Bubbletimer.bubbletimer_history_backward_array.pop();
};


/**
 * Save a slot to into the redo (future) array
 *
 * @param {Object} history
 */
Drupal.Bubbletimer._bubbletimer_save_forward = function (history) {
  Drupal.Bubbletimer.bubbletimer_history_forward_array.push(history);
};


/**
 * Get an already undo-d action from the redo (future) array
 * @return {Object}
 */
Drupal.Bubbletimer._bubbletimer_get_forward = function () {
  return Drupal.Bubbletimer.bubbletimer_history_forward_array.pop();
};

Drupal.ajax.prototype.commands.reinit_date_selector_form = function () {
  Drupal.Bubbletimer.bt_init_table();
  Drupal.Bubbletimer.init_date_selector_form();
};

Drupal.ajax.prototype.commands.activate_hide_link = function (ajax, response, status) {
  $("#bubbletimer-ahah-hide-link").click(function (event) {
    event.preventDefault();
    $("#bubbletimer").remove();
    $("#bubbletimer-ahah-hide-link")
      .before(response.link)
      .remove();
    Drupal.behaviors.AJAX.attach(document, Drupal.settings);
  });
};

Drupal.ajax.prototype.commands.reinit = function () {
  Drupal.behaviors.AJAX.attach(document, Drupal.settings);
  Drupal.Bubbletimer.ajaxform.init();
};

Drupal.Bubbletimer.ajaxform.init = function() {
  var forms = [
    "#bubbletimer-date-selector-form",
    "#bubbletimer-activityform"
  ];
  for (var i in forms) {
    $(forms[i])
      .not('.bubbletimer-ajax-processed')
      .addClass('bubbletimer-ajax-processed')
      .submit(function (event) {
        event.preventDefault();
        var formdata = $(this).serialize();
        formdata += '&op=' + escape($("#edit-bt-submit", $(this)).val());
        $.ajax({
          type: 'post',
          url: $(this).attr('action'),
          data: formdata,
          success: Drupal.Bubbletimer.ajaxform.success,
          dataType: 'json'
        });
        return false;
      });
  }
};

Drupal.Bubbletimer.ajaxform.success = function (response, status) {
  console.log(response);
  Drupal.ajax.prototype.success(response);
};

/**
 * Update statistics
 * It has a 3 seconds delay to prevent being blocked by google-chart
 *  - time sums
 *  - google chart
 * @return
 */
Drupal.Bubbletimer.bt_update_stats = function () {
  // Update stats
  Drupal.Bubbletimer.bt_throbber_on();
  clearTimeout(Drupal.Bubbletimer.bt_update_delay);
  if (Drupal.Bubbletimer.bt_first_reload) {
    Drupal.Bubbletimer._bt_update_stats_batch();
    Drupal.Bubbletimer.bt_first_reload = false;
  }
  else {
    Drupal.Bubbletimer.bt_update_delay =
        setTimeout(Drupal.Bubbletimer._bt_update_stats_batch,
            Drupal.Bubbletimer.BT_UPDATE_DELAY_MS);
  }
};


/**
 * Call updaters
 * @return
 */
Drupal.Bubbletimer._bt_update_stats_batch = function () {
  Drupal.Bubbletimer.bt_update_activity_percent();
  if($("#bt_chart_placeholder").length) {
    Drupal.Bubbletimer.bt_update_chart();
  }
};


/**
 * Update node titles in the timesheet, how much percent of usage those has
 */
Drupal.Bubbletimer.bt_update_activity_percent = function () {
  var slot_array = $('table#bt_table input:hidden[value="1"]');
  var nid_selected_array = [];
  $.each(slot_array, function() {
    var nid = $(this).attr('nid');
    if (nid) {
      if (!nid_selected_array[nid]) {
        nid_selected_array[nid] = 0;
      }
      nid_selected_array[nid]++;
    }
  });
  $.each(Drupal.Bubbletimer.bt_job_array, function () {
    var hours = nid_selected_array[this] * 0.25;
    $('span#bt_node_percent_' + this).html((hours || 0).toString());
  });

  $('span.bt_sum_hour').html(((slot_array.length * 0.25) || 0).toString());
};


/**
 * Update google chart
 * @return
 */
Drupal.Bubbletimer.bt_update_chart = function () {
  if (!Drupal.settings.bubbletimer.piechart_enable &&
      !Drupal.settings.bubbletimer.barchart_enable) {
    Drupal.Bubbletimer.bt_throbber_off();
    return;
  }

  var url_pie = 'http://chart.apis.google.com/chart?chs=600x100&cht=p3&chco=0072b9,E8B305,E8003A,6DE809&';
  var url_bar = 'http://chart.apis.google.com/chart?cht=bvg&chco=FF2A2A,FFBE33,AAFF33,36FFE1,365BFF,AE35FF,FF36A1,FF8383,FFD884,BAFF81,84FFD0,88B4FF,BB84FF,FF88DD&chds=0,8&chxt=y&chxr=0,0,8,2&';
  var percent = [];
  var total = [];
  var title = [];
  var times = [];
  var sum = 0;

  $.each($('td.bt_title_td'), function () {
    var time = parseFloat($('span', this).html());
    if (time > 0) {
      title.push(encodeURIComponent($('a:last', this).html()));
      times.push(time);
      sum += time;
    }
  });

  if (times.length == 0) {
    Drupal.Bubbletimer.bt_throbber_off();
    $('div#bt_chart_placeholder').html('');
    return;
  }

  $.each(times, function () {
    percent.push(100 * (this / sum));
    total.push(this);
  });

  var percent_url_pie = percent.join(',');
  var total_url_bar   = total.join('|');
  var title_url       = title.join('|');

  url_pie = url_pie + 'chd=t:' + percent_url_pie + '&chl=' + title_url;
  url_bar = url_bar + 'chs=540x' + (title.length * 18 > 100 ? title.length * 18 : 100) + '&chd=t:' + total_url_bar + '&chdl=' + title_url;
  var pie_img = Drupal.settings.bubbletimer.piechart_enable ? '<img src="'+ url_pie + '" />' : '';
  var bar_img = Drupal.settings.bubbletimer.barchart_enable ? '<img src="'+ url_bar + '" />' : '';
  $('div#bt_chart_placeholder').html(pie_img + bar_img);
  $('div#bt_chart_placeholder img').load(function () {bt_throbber_off();});
};

})(jQuery);
