<?php


/**
 * @file
 * Bubbletimer main page
 */

/**
 * Bubbletimer interface
 * Render
 * 	- title
 * 	- date selector form part
 * 	- activity form
 * @return string
 */
function bubbletimer_page($js = FALSE, $nid = NULL) {
  $date_form = drupal_get_form('bubbletimer_date_selector_form');

  $user = bubbletimer_get_user();

  drupal_set_title(t('Bubbletimer - @username\'s daily basis',
    array('@username' => $user->name)));

  $nodes_enabled = $nid === NULL ?
      bubbletimer_get_enabled_jobs(bubbletimer_get_user()) :
      bubbletimer_ahah_check_nid($nid);

  $node_form = $js ? '' :
    drupal_render(drupal_get_form('bubbletimer_node_add_form', count($nodes_enabled) == 0));

  if (!empty($nodes_enabled)) {
    $bubbletimer_form =
      drupal_render(drupal_get_form('bubbletimer_activityform', $nodes_enabled, !$js));
  }
  else {
    $bubbletimer_form = '';
  }

  $output = theme('bubbletimer_page', array(
    'date_form' => drupal_render($date_form),
    'node_form' => $node_form,
    'bubbletimer_form' => $bubbletimer_form,
  ));
  if ($js === TRUE || $js === 'ajax') {
    $commands = array();
    if ($nid !== NULL) {
      $commands[] = ajax_command_after('#bubbletimer-ahah-link',
        theme('bubbletimer_ahah_hide_link', array('nid' => $nid)));
    }
    $commands[] = ajax_command_remove('#bubbletimer');
    $commands[] = ajax_command_before('#bubbletimer-remove-link', $output);
    $commands[] = ajax_command_remove('#bubbletimer-ahah-link');
    $commands[] = ajax_command_reinit_date_selector_form();
    $commands[] =
      ajax_command_activate_hide_link(theme_bubbletimer_ahah_show_link($nid));
    $commands[] = ajax_command_reinit();

    ajax_deliver(array(
      '#type' => 'ajax',
      '#commands' => $commands,
    ));
  }
  else {
    bubbletimer_include_js_css();
    return $output;
  }
}


/**
 * Definition of activity form
 *
 * @return array
 */
function bubbletimer_activityform($form, &$form_state, array $nids, $quickchart = TRUE) {
  $user = bubbletimer_get_user();
  $timestamp_original = bubbletimer_get_timestamp();

  $f = array();

  $f['#quickchart'] = $quickchart;

  $f['#theme'] = 'bt_form';

  $f['bt_wrapper'] = array(
    '#theme' => 'bt_wrapper',
  );

  // Checkboxes and titles / node / step
  foreach ($nids as $nid => $node_title) {
    $timestamp = $timestamp_original;

    $f['bt_wrapper']['title'][$nid] = array(
      '#markup' => l(strlen($node_title) > BT_TITLE_LENGTH ?
        substr($node_title, 0, BT_TITLE_LENGTH) . '...' : $node_title,
        'node/' . $nid, array(
          'absolute' => TRUE,
          'attributes' => array(
            'class' => 'job_node',
            'nid' => $nid,
          ),
          'html' => TRUE,
        )) . "<span class=\"bt_small\">,
          <span id=\"bt_node_percent_{$nid}\"></span>h</span>",
      '#nid' => $nid,
    );

    $nid_pattern = bubbletimer_get_job_pattern($user->uid, $nid,
      date('Y-m-d', bubbletimer_get_timestamp()));

    for ($i = 0; $i < BT_STAGE_COUNT; $i++) {
      $f['bt_wrapper']['slot'][$nid][$timestamp] = array(
        '#type' => 'bubbletimer_slot',
        '#default_value' => substr($nid_pattern, $i, 1) == '1' ? 1 : 0,
      );
      $timestamp += BT_SECONDS_INTERVAL;
    }
  }

  $f['actions'] = array('#type' => 'actions');

  // Submit button
  $f['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save changes'),
    // @todo these should be different form items with custom templates
    '#suffix' => '
    	<input type="button" onclick="Drupal.Bubbletimer.bubbletimer_history_backward()" value="' . t('Undo') . '" />
    	<input type="button" onclick="Drupal.Bubbletimer.bubbletimer_history_forward()" value="' . t('Redo') . '" />
    ',
  );

  return $f;
}


/**
 * Define date selector form
 * @param array $form_state information about form
 * @return string
 */
function bubbletimer_date_selector_form($form, &$form_state) {
  $f = array();

  $f['bt_date'] = array(
    '#title' => t('Go to'),
    '#type' => 'date',
    '#default_value' => array(
      'year' => date('Y'),
      'month' => date('m'),
      'day' => date('d'),
    ),
  );

  $f['bt_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  $f['bt_date_last'] = array(
    '#type' => 'button',
    '#value' => t('Last edited day'),
  );

  $f['bt_date_now'] = array(
    '#type' => 'button',
    '#value' => t('Today'),
  );

  return $f;
}


/**
 * Assign or unassign a certain node to a user -> to get it into the timesheet list
 * @param integer $nid
 * @param boolean $is_assign
 */
function bubbletimer_assign_node_to_user($nid, $is_assigned, $silent = FALSE) {
  $user = bubbletimer_get_user();

  $count = db_query('SELECT COUNT(*) FROM {bubbletimer_node_user}
    WHERE uid = :uid AND nid = :nid', array(
      ':uid' => $user->uid,
      ':nid' => $nid,
    ))->fetchField();

  if ($is_assigned && $count == 0) {
    db_insert('bubbletimer_node_user')
      ->fields(array(
        'uid' => $user->uid,
        'nid' => $nid,
      ))
      ->execute();
    if (!$silent) {
      drupal_set_message(t('This node will appear in your timesheet list.'));
    }
  }
  elseif (!$is_assigned && $count > 0) {
    db_delete('bubbletimer_node_user')
      ->condition('uid', $user->uid)
      ->condition('nid', $nid)
      ->execute();
    if (!$silent) {
      drupal_set_message(t('Node is deleted from the timesheet list.'));
    }
  }
}


/**
 * Implementation of hook_submit()
 * Saving selected day in session
 * @param array $form
 * @param arary $form_state
 */
function bubbletimer_date_selector_form_submit($form, $form_state) {
  $bt_date = $form_state['values']['bt_date'];
  $datearr = explode('-',
    date('Y-m-d', (!empty($bt_date) ? strtotime($bt_date) : time())));
  bubbletimer_set_date(array(
    'year' => $datearr[0],
    'month' => $datearr[1],
    'day' => $datearr[2]
  ));
}


/**
 * Implmentation of hook_submit()
 * Saving daily activities into database
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_activityform_submit($form, $form_state) {
  $user = bubbletimer_get_user();

  foreach ($form_state['clicked_button']['#post']['slot'] as $nid => $form_row) {
    // @TODO use #tree instead of this hackish access
    $activity = '';
    foreach ((array)$form_row as $timestamp => $value) {
      $activity .= $value == '1' ? '1' : '0'; // Sanitizing
    }
    db_merge('bubbletimer_activity')
      ->fields(array(
        'activity' => $activity,
      ))
      ->key(array(
        'date' => implode('-', bubbletimer_get_date()),
        'uid' => $user->uid,
        'nid' => $nid,
      ))
      ->execute();

    // If node updating is enabled.
    // Ex.: Computed field needs node update to recalculate data.
    if (variable_get('bubbletimer_node_refresh', '0') == '1') {
      $node = node_load($nid);
      node_save($node);
    }
  }
}


/**
 * Implementation of theme_hook()
 *
 * Just for adding a submit right next to the date selectors
 *
 * @param array $element
 * @return string
 */
function theme_bubbletimer_date_field($variables) {
  $element = $variables['element'];
  return drupal_render($element) . '<input type="submit" value="Go" />';
}


/**
 * Implementation of hook_form_alter()
 *
 * Setup forms:
 * 	- Set activity in the form (from datas in bubbletimer table)
 * 	- Set the current date on date selector form
 *
 * @param array $form
 * @param array $form_state
 * @param string $form_id
 */
function bubbletimer_form_alter(&$form, &$form_state, $form_id) {
  if (empty($_POST)) {
    switch ($form_id) {
      case 'bubbletimer_date_selector_form':
        $date_array = array_map('intval', bubbletimer_get_date());
        $form['bt_date']['#value'] = format_date(strtotime(
          "{$date_array['year']}-{$date_array['month']}-{$date_array['day']}"),
          'custom', 'm/d/Y');
        break;
      case 'bubbletimer_node_assign_form':
        $user = bubbletimer_get_user();
        $form['bubbletimer_node_assign_fieldset']['bt_assign_checkbox']['#value'] =
        bubbletimer_is_node_assigned_to_user($user->uid,
          $form['bubbletimer_node_assign_fieldset']['bt_node_nid']['#value']);
        break;
    }
  }
}

/**
 * Form for adding nodes to bt on bubbletimer's page
 *
 * @param array $form_state
 * @param boolean $enable_nodes_empty - if no nodes enables, it gets not collapsed
 * @return array
 */
function bubbletimer_node_add_form($form, &$form_state, $enable_nodes_empty) {
  $nodes_available = bubbletimer_get_jobs_available();
  $node_types = node_type_get_types();
  $node_types_available = array();
  foreach ($nodes_available as $node_type => $node) {
    $node_types_available[$node_type] = $node_types[$node_type]->name;
  }

  $form['bt_nodeadd_fieldset'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => !$enable_nodes_empty,
    '#title' => t('Add new node to the timesheet'),
  );
  if (!empty($node_types_available)) {
    $form['bt_nodeadd_fieldset']['bt_node_type'] = array(
      '#type' => 'select',
      '#title' => t('Node type'),
      '#options' => $node_types_available,
    );
  }
  elseif (!empty($nodes_available) && user_access(BT_PERM_ADMIN)) {
    $form['bt_nodeadd_fieldset']['bt_info'] = array(
      '#type' => 'item',
      '#title' => t('No content types available for Bubbletimer. ' .
        'Set on the <a href="@link">Bubbletimer settings</a> page.',
              array('@link' => url('admin/config/content/bubbletimer'))),
    );
  }
  foreach ($nodes_available as $node_type => $nodes) {
    $form['bt_nodeadd_fieldset']['bt_addnode_select_'.$node_type] = array(
      '#type' => 'select',
      '#title' => $node_types_available[$node_type],
      '#options' => $nodes,
      '#attributes' => array(
        'class' => array('bubbletimer_node_select'),
        'node_type' => $node_type,
      ),
    );
  }
  $form['bt_nodeadd_fieldset']['bt_addnode_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
  );

  return $form;
}


/**
 * Submit handler for node-adding form
 *
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_node_add_form_submit($form, $form_state) {
  bubbletimer_assign_node_to_user(intval($form_state['values']['bt_addnode_select_' .
    $form_state['values']['bt_node_type']]), TRUE);
  drupal_goto(BT_PATH);
}

function theme_bt_wrapper($variables) {
  global $base_path;
  $path = $base_path . drupal_get_path('module', 'bubbletimer');

  $form = $variables['element'];

  $nodes_assigned = bubbletimer_get_jobs_assigned(bubbletimer_get_user());
  $nodes_assigned_keys = array_keys($nodes_assigned);

  $table_rows = array();
  $row = array();
  $row[] = array(
    'data' => '<span class="wait bt_chart_updater"></span>
      Projects <span class="bt_small">(<span class="bt_sum_hour"></span>h a day)</span>',
    'class' => 'bt_td_header',
  );

  $form_keys = array_keys($form['slot']);
  $nids = array_filter($form_keys, 'is_numeric');

  // Left arrow
  $row[] = array(
    'data' => "<img src=\"{$path}/images/step_left.gif\" alt=\"step left\" />",
    'rowspan' => count($nids) + 1,
    'id' => 'bt_table_step_left',
  );

  for ($hour = BT_START_HOUR; $hour < BT_END_HOUR; $hour++) {
    $row[] = array(
      'data' => $hour.'h',
      'colspan' => BT_SLOTS_PER_HOUR,
      'hour' => $hour,
      'class' => 'bt_td_header bt_hour_td ' . ($hour & 1 ? 'even' : 'odd'),
      'style' => $hour < 8 || $hour >= 18 ? 'display:none;' : '',
    );
  }

  // Right arrow
  $row[] = array(
    'data' => "<img src=\"{$path}/images/step_right.gif\" alt=\"step right\" />",
    'rowspan' => count($nids) + 1,
    'id' => 'bt_table_step_right',
  );
  $table_rows[] = $row;

  foreach ($form['slot'] as $nid => $form_row) {
    if (!is_array($form_row) || !is_numeric($nid)) continue;
    $row = array();

    $title = array_shift($form['title']);
    $assign_link = in_array($nid, $nodes_assigned_keys) ?
      l('&nbsp;', BT_URL_NODE_REMOVE . '/' . $nid, array(
        'attributes' => array('class' => 'bubbletimer_node_remove'),
        'html' => TRUE,
      )):
      l('&nbsp;', BT_URL_NODE_ADD . '/' . $nid, array(
        'attributes' => array('class' => 'bubbletimer_node_add'),
        'html' => TRUE,
      ));
    $row[] = array(
      'data' => $assign_link . drupal_render($title),
      'class' => 'bt_title_td',
      'nid' => $nid,
    );

    $hour = BT_START_HOUR;
    foreach ($form_row as $timestamp => $element) {
      if (!is_numeric($timestamp)) continue;
      $row[] = array(
        'data' => drupal_render($element),
        'hour' => floor($hour),
        'class' => 'bt_slot_td ' . (floor($hour) & 1 ? 'even' : 'odd'),
        'valign' => 'bottom',
        'nid' => $nid,
        'style' => $hour < 8 || $hour >= 18 ? 'display:none;' : '',
      );
      $hour += 1 / BT_SLOTS_PER_HOUR;
    }
    $table_rows[] = $row;
  }

  return theme('table', array(
    'rows' => $table_rows,
    'attributes' => array(
      'id' => 'bt_table',
    ),
  ));
}


/**
 * Node assign form.
 *
 * @param array $form_original
 * @param int $nid
 * @param string $action
 * @return array
 */
function bubbletimer_assign_node_form($form, &$form_state, $nid, $action = BT_NODE_REMOVE) {
  drupal_set_title(t('Bubbletimer - @action node',
    array('@action' => $action == BT_NODE_ADD ? t('add') : t('remove'))));
  $node = node_load($nid);

  return array(
    'text' => array(
      '#type' => 'item',
      '#title' => t('Do you want to @action the node: @title?', array(
        '@action' => $action == BT_NODE_ADD ? t('add') : t('remove'),
        '@title' => $node->title,
      )),
    ),
    'nid' => array(
      '#type' => 'hidden',
      '#value' => (int) $nid,
    ),
    'action' => array(
      '#type' => 'hidden',
      '#value' => $action,
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => $action == BT_NODE_ADD ? t('Add node') : t('Remove node'),
    ),
  );
}


/**
 * Submission for node assign form.
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_assign_node_form_submit($form, $form_state) {
  bubbletimer_assign_node_to_user(
    $form_state['values']['nid'], $form_state['values']['action'] == BT_NODE_ADD);
  drupal_goto('bubbletimer');
}

function bubbletimer_ahah_check_nid($nid) {
  global $user;
  $node = node_load($nid);
  if ($node === FALSE || !node_access('view', $node)) return array();
  return db_query('SELECT COUNT(*) FROM {bubbletimer_node_user}
    WHERE uid = :uid AND nid = :nid', array(
      ':uid' => $user->uid,
      ':nid' => $nid,
    )) ?
    array($node->nid => $node->title):
    array();
}

function bubbletimer_add_node($js, $node) {
  bubbletimer_assign_node_to_user($node->nid, TRUE, TRUE);

  $commands = array();
  $commands[] = ajax_command_before("#bubbletimer-add-link",
    bubbletimer_theme_wrapper(
      theme('bubbletimer_ahah_show_link', array('nid' => $node->nid))));
  $commands[] = ajax_command_before("#bubbletimer-add-link",
    bubbletimer_theme_wrapper(
      theme('bubbletimer_remove_link', array('nid' => $node->nid))));
  $commands[] = ajax_command_remove("#bubbletimer-add-link");

  bubbletimer_crud_redirect_handler($js, $node, $commands);
}

function bubbletimer_del_node($js, $node) {
  bubbletimer_assign_node_to_user($node->nid, FALSE, TRUE);
  $link = bubbletimer_theme_wrapper(
    theme('bubbletimer_add_link', array('nid' => $node->nid)));

  $commands = array();
  $commands[] = ajax_command_before("#bubbletimer-ahah-link", $link);
  $commands[] = ajax_command_before("#bubbletimer-ahah-hide-link", $link);
  $commands[] = ajax_command_remove("#bubbletimer-ahah-link");
  $commands[] = ajax_command_remove("#bubbletimer-ahah-hide-link");
  $commands[] = ajax_command_remove("#bubbletimer");
  $commands[] = ajax_command_remove("#bubbletimer-remove-link");

  bubbletimer_crud_redirect_handler($js, $node, $commands);
}

function bubbletimer_crud_redirect_handler($js, $node, $commands) {
  if ($js) {
    $commands[] = ajax_command_reinit();
    ajax_deliver(array(
      '#type' => 'ajax',
      '#commands' => $commands,
    ));
  }
  else {
    drupal_goto('node/' . $node->nid);
  }
}
