<?php

/**
 * Returns a list of all nodealiases.
 *
 * @param int $uid
 * @return
 */
function bubbletimer_nodealias_list($uid) {
  return db_query('SELECT * FROM {bubbletimer_node_alias} btn JOIN {node} n USING (nid)
    WHERE btn.uid = :uid', array(
      ':uid' => $uid,
    ))->fetchAll();
}

/**
 * Fetches a node alias from the database.
 *
 * @param int $uid
 * @param int $nid
 * @return string
 */
function bubbletimer_nodealias_load($uid, $nid) {
  return db_query('SELECT "alias" FROM {bubbletimer_node_alias}
    WHERE uid = :uid AND nid = :nid', array(
      ':uid' => $uid,
      ':nid' => $nid,
    ))->fetchField();
}

/**
 * Returns a node id based on the user id and alias.
 *
 * @param int $uid
 * @param string $alias
 * @return int
 */
function bubbletimer_nodealias_load_by_alias($uid, $alias) {
  return db_query('SELECT nid FROM {bubbletimer_node_alias}
    WHERE uid = :uid AND "alias" = :alias', array(
      ':uid' => $uid,
      ':alias' => $alias,
    ))->fetchField();
}

/**
 * Inserts or updates an alias to the database.
 *
 * @param int $uid
 * @param int $nid
 * @param string $alias
 */
function bubbletimer_nodealias_save($uid, $nid, $alias) {
  db_merge('bubbletimer_node_alias')
    ->key(array(
      'uid' => $uid,
      'nid' => $nid,
    ))
    ->fields(array(
      'alias' => $alias,
    ))
    ->execute();
}

/**
 * Deletes an alias.
 *
 * @param int $uid
 * @param int $nid
 */
function bubbletimer_nodealias_delete($uid, $nid) {
  db_delete('bubbletimer_node_alias')
    ->condition('uid', $uid)
    ->condition('nid', $nid)
    ->execute();
}
